@@ -18,10 +18,10 @@
 #   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from gi.repository import Gtk, GObject, Gio, GdkPixbuf, Gdk, GMenu, GLib
-import cgi, os
+import cgi
+import os
 import gettext
 import subprocess
-import urllib
 
 from Alacarte import config
 gettext.bindtextdomain(config.GETTEXT_PACKAGE, config.localedir)
@@ -34,14 +34,13 @@ from Alacarte import util
 class MainWindow(object):
     timer = None
     #hack to make editing menu properties work
-    allow_update = True
     edit_pool = []
 
     def __init__(self, datadir, version):
         self.file_path = datadir
         self.version = version
         self.editor = MenuEditor()
-        self.editor.applications.tree.connect("changed", self.menuChanged)
+        self.editor.tree.connect("changed", self.menuChanged)
         Gtk.Window.set_default_icon_name('alacarte')
         self.tree = Gtk.Builder()
         self.tree.set_translation_domain(config.GETTEXT_PACKAGE)
@@ -69,8 +68,6 @@ class MainWindow(object):
         self.loadUpdates()
 
     def loadUpdates(self):
-        if not self.allow_update:
-            return False
         menu_tree = self.tree.get_object('menu_tree')
         item_tree = self.tree.get_object('item_tree')
         items, iter = item_tree.get_selection().get_selected()
@@ -85,7 +82,7 @@ class MainWindow(object):
             elif isinstance(items[iter][3], GMenu.TreeDirectory):
                 item_id = os.path.split(items[iter][3].get_desktop_file_path())[1]
                 update_type = GMenu.TreeItemType.DIRECTORY
-            elif isinstance(items[iter][3], GMenu.Tree.Separator):
+            elif isinstance(items[iter][3], GMenu.TreeSeparator):
                 item_id = items.get_path(iter)
                 update_type = GMenu.TreeItemType.SEPARATOR
         menus, iter = menu_tree.get_selection().get_selected()
@@ -220,7 +217,7 @@ class MainWindow(object):
                 name = item.get_name()
             elif isinstance(item, GMenu.TreeEntry):
                 name = item.get_app_info().get_display_name()
-            elif isinstance(item, GMenu.Separator):
+            elif isinstance(item, GMenu.TreeSeparator):
                 name = '---'
             else:
                 assert False, 'should not be reached'
@@ -439,12 +436,7 @@ class MainWindow(object):
             return
         item = items[path][3]
         before = items[(path.get_indices()[0] - 1,)][3]
-        if isinstance(item, GMenu.TreeEntry):
-            self.editor.moveItem(item, item.get_parent(), before=before)
-        elif isinstance(item, GMenu.TreeDirectory):
-            self.editor.moveMenu(item, item.get_parent(), before=before)
-        elif isinstance(item, GMenu.TreeSeparator):
-            self.editor.moveSeparator(item, item.get_parent(), before=before)
+        self.editor.moveItem(item.get_parent(), item, before=before)
 
     def on_move_down_button_clicked(self, button):
         item_tree = self.tree.get_object('item_tree')
@@ -457,12 +449,7 @@ class MainWindow(object):
             return
         item = items[path][3]
         after = items[path][3]
-        if isinstance(item, GMenu.TreeEntry):
-            self.editor.moveItem(item, item.get_parent(), after=after)
-        elif isinstance(item, GMenu.TreeDirectory):
-            self.editor.moveMenu(item, item.get_parent(), after=after)
-        elif isinstance(item, GMenu.TreeSeparator):
-            self.editor.moveSeparator(item, item.get_parent(), after=after)
+        self.editor.moveItem(item.get_parent(), item, after=after)
 
     def on_help_button_clicked(self, *args):
         Gtk.show_uri(Gdk.Screen.get_default(), "ghelp:user-guide#menu-editor", Gtk.get_current_event_time())
@@ -483,8 +470,5 @@ class MainWindow(object):
     def on_delete_button_clicked(self, button):
         self.on_edit_delete_activate(None)
 
-    def on_style_set(self, *args):
-        self.loadUpdates()
-
     def quit(self):
         Gtk.main_quit()
