@@ -22,12 +22,12 @@ import xml.parsers.expat
 from gi.repository import GMenu, GLib
 from Alacarte import util
 
-class Menu(object):
-    def __init__(self, name):
+class MenuEditor(object):
+    def __init__(self, name='applications.menu'):
         self.name = name
 
         self.tree = GMenu.Tree.new(name, GMenu.TreeFlags.SHOW_EMPTY|GMenu.TreeFlags.INCLUDE_EXCLUDED|GMenu.TreeFlags.INCLUDE_NODISPLAY|GMenu.TreeFlags.SHOW_ALL_SEPARATORS|GMenu.TreeFlags.SORT_DISPLAY_NAME)
-        self.visible_tree = GMenu.Tree.new(name, GMenu.TreeFlags.SORT_DISPLAY_NAME)
+        self.tree.connect('changed', self.menuChanged)
         self.load()
 
         self.path = os.path.join(util.getUserMenuPath(), self.tree.props.menu_basename)
@@ -43,31 +43,24 @@ class Menu(object):
     def load(self):
         if not self.tree.load_sync():
             raise ValueError("can not load menu tree %r" % (self.name,))
-        if not self.visible_tree.load_sync():
-            raise ValueError("can not load menu tree %r" % (self.name,))
-
-class MenuEditor(object):
-    def __init__(self):
-        self.applications = Menu('applications.menu')
-        self.applications.tree.connect('changed', self.menuChanged)
 
     def menuChanged(self, *a):
-        self.applications.load()
+        self.load()
 
     def save(self):
-        fd = open(self.applications.path, 'w')
-        fd.write(self.applications.dom.toprettyxml())
+        fd = open(self.path, 'w')
+        fd.write(self.dom.toprettyxml())
         fd.close()
 
     def revert(self):
-        self.revertTree(self.applications.tree.get_root_directory())
-        path = os.path.join(util.getUserMenuPath(), os.path.basename(self.applications.tree.get_canonical_menu_path()))
+        self.revertTree(self.tree.get_root_directory())
+        path = os.path.join(util.getUserMenuPath(), os.path.basename(self.tree.get_canonical_menu_path()))
         try:
             os.unlink(path)
         except OSError:
             pass
 
-        self.applications.loadDOM()
+        self.loadDOM()
         self.save()
 
     def revertTree(self, menu):
@@ -106,9 +99,10 @@ class MenuEditor(object):
             pass
         self.save()
 
-    def getMenus(self, parent=None):
+    def getMenus(self, parent):
         if parent is None:
-            parent = self.applications.tree.get_root_directory()
+            yield (self.tree.get_root_directory(), True)
+            return
 
         item_iter = parent.iter()
         item_type = item_iter.next()
@@ -124,6 +118,7 @@ class MenuEditor(object):
         item_type = item_iter.next()
 
         while item_type != GMenu.TreeItemType.INVALID:
+            item = None
             if item_type == GMenu.TreeItemType.DIRECTORY:
                 item = item_iter.get_directory()
             elif item_type == GMenu.TreeItemType.ENTRY:
@@ -132,6 +127,8 @@ class MenuEditor(object):
                 item = item_iter.get_header()
             elif item_type == GMenu.TreeItemType.ALIAS:
                 item = item_iter.get_alias()
+            elif item_type == GMenu.TreeItemType.SEPARATOR:
+                item = item_iter.get_separator()
             if item:
                 contents.append(item)
             item_type = item_iter.next()
@@ -141,19 +138,18 @@ class MenuEditor(object):
         item_iter = menu.iter()
         item_type = item_iter.next()
         while item_type != GMenu.TreeItemType.INVALID:
-            if item_type == GMenu.TreeItemType.SEPARATOR:
+            item = None
+            if item_type == GMenu.TreeItemType.ENTRY:
+                item = item_iter.get_entry()
+            elif item_type == GMenu.TreeItemType.DIRECTORY:
+                item = item_iter.get_directory()
+            elif item_type == GMenu.TreeItemType.HEADER:
+                item = item_iter.get_header()
+            elif item_type == GMenu.TreeItemType.ALIAS:
+                item = item_iter.get_alias()
+            elif item_type == GMenu.TreeItemType.SEPARATOR:
                 item = item_iter.get_separator()
-                yield (item, True)
-            else:
-                if item_type == GMenu.TreeItemType.ENTRY:
-                    item = item_iter.get_entry()
-                elif item_type == GMenu.TreeItemType.DIRECTORY:
-                    item = item_iter.get_directory()
-                elif item_type == GMenu.TreeItemType.HEADER:
-                    item = item_iter.get_header()
-                elif item_type == GMenu.TreeItemType.ALIAS:
-                    item = item_iter.get_alias()
-                yield (item, self.isVisible(item))
+            yield (item, self.isVisible(item))
             item_type = item_iter.next()
 
     def canRevert(self, item):
@@ -174,7 +170,7 @@ class MenuEditor(object):
         return False
 
     def setVisible(self, item, visible):
-        dom = self.applications.dom
+        dom = self.dom
         if isinstance(item, GMenu.TreeEntry):
             menu_xml = self.getXmlMenu(self.getPath(item.get_parent()), dom.documentElement, dom)
             if visible:
@@ -202,7 +198,7 @@ class MenuEditor(object):
 
     def insertExternalItem(self, file_id, parent_id, before=None, after=None):
         parent = self.findMenu(parent_id)
-        dom = self.applications.dom
+        dom = self.dom
         self.addItem(parent, file_id, dom)
         self.positionItem(parent, ('Item', file_id), before, after)
         self.save()
@@ -210,7 +206,7 @@ class MenuEditor(object):
     def insertExternalMenu(self, file_id, parent_id, before=None, after=None):
         menu_id = file_id.rsplit('.', 1)[0]
         parent = self.findMenu(parent_id)
-        dom = self.applications.dom
+        dom = self.dom
         self.addXmlDefaultLayout(self.getXmlMenu(self.getPath(parent), dom.documentElement, dom) , dom)
         menu_xml = self.getXmlMenu(self.getPath(parent) + [menu_id], dom.documentElement, dom)
         self.addXmlTextElement(menu_xml, 'Directory', file_id, dom)
@@ -231,7 +227,7 @@ class MenuEditor(object):
             parent = item.get_parent()
         self.writeItem(item, Icon=icon, Name=name, Comment=comment, Exec=command, Terminal=use_term)
         if final:
-            dom = self.applications.dom
+            dom = self.dom
             menu_xml = self.getXmlMenu(self.getPath(parent), dom.documentElement, dom)
             self.addXmlTextElement(menu_xml, 'AppDir', util.getUserItemPath(), dom)
         self.save()
@@ -242,7 +238,7 @@ class MenuEditor(object):
             return
         #we don't use this, we just need to make sure the <Menu> exists
         #otherwise changes won't show up
-        dom = self.applications.dom
+        dom = self.dom
         menu_xml = self.getXmlMenu(self.getPath(menu), dom.documentElement, dom)
         self.writeMenu(menu, Icon=icon, Name=name, Comment=comment)
         if final:
@@ -250,7 +246,7 @@ class MenuEditor(object):
         self.save()
 
     def copyItem(self, item, new_parent, before=None, after=None):
-        dom = self.applications.dom
+        dom = self.dom
         file_path = item.get_desktop_file_path()
         keyfile = GLib.KeyFile()
         keyfile.load_from_file(file_path, util.KEY_FILE_FLAGS)
@@ -272,52 +268,12 @@ class MenuEditor(object):
         self.save()
         return file_id
 
-    def moveItem(self, item, new_parent, before=None, after=None):
-        if item.get_parent() != new_parent:
-            #hide old item
-            self.deleteItem(item)
-            file_id = self.copyItem(item, new_parent)
-            item = ('Item', file_id)
-        self.positionItem(new_parent, item, before, after)
-        self.save()
-
-    def moveMenu(self, menu, new_parent, before=None, after=None):
-        parent = new_parent
-        #don't move a menu into it's child
-        while parent.get_parent():
-            parent = parent.get_parent()
-            if parent == menu:
-                return False
-
-        #don't move a menu into itself
-        if new_parent == menu:
-            return False
-
-        if menu.get_parent() != new_parent:
-            dom = self.applications.dom
-            path = self.getPath(menu)
-            root_path = path[0]
-            xml_root = self.getXmlMenu(root_path, dom.documentElement, dom)
-            old_path = path[1:]
-            new_path = self.getPath(new_parent)[1:] + [menu.get_menu_id()]
-            self.addXmlMove(xml_root, '/'.join(old_path), '/'.join(new_path), dom)
-        self.positionItem(new_parent, menu, before, after)
-        self.save()
-
-    def moveSeparator(self, separator, new_parent, before=None, after=None):
-        # remove the original separator if its parent is not the new destination
-        if separator.get_parent() != new_parent:
-            self.deleteSeparator(separator)
-        # this adds the new separator to the specified position
-        self.positionItem(new_parent, separator, before, after)
-        self.save()
-
     def deleteItem(self, item):
         self.writeItem(item, Hidden=True)
         self.save()
 
     def deleteMenu(self, menu):
-        dom = self.applications.dom
+        dom = self.dom
         menu_xml = self.getXmlMenu(self.getPath(menu), dom.documentElement, dom)
         self.addDeleted(menu_xml, dom)
         self.save()
@@ -327,14 +283,14 @@ class MenuEditor(object):
         contents = self.getContents(parent)
         contents.remove(item)
         layout = self.createLayout(contents)
-        dom = self.applications.dom
+        dom = self.dom
         menu_xml = self.getXmlMenu(self.getPath(parent), dom.documentElement, dom)
         self.addXmlLayout(menu_xml, layout, dom)
         self.save()
 
     def findMenu(self, menu_id, parent=None):
         if parent is None:
-            parent = self.applications.tree.get_root_directory()
+            parent = self.tree.get_root_directory()
 
         if menu_id == parent.get_menu_id():
             return parent
@@ -355,11 +311,8 @@ class MenuEditor(object):
         if isinstance(item, GMenu.TreeEntry):
             app_info = item.get_app_info()
             return not (item.get_is_excluded() or app_info.get_nodisplay())
-
-        root = self.applications.visible_tree.get_root_directory()
-        if isinstance(item, GMenu.TreeDirectory):
-            if self.findMenu(item.get_menu_id(), root) is None:
-                return False
+        elif isinstance(item, GMenu.TreeDirectory):
+            return not item.get_is_nodisplay()
         return True
 
     def getPath(self, menu):
@@ -497,7 +450,7 @@ class MenuEditor(object):
 
         # add new layout
         node = dom.createElement('Layout')
-        for order in layout.order:
+        for order in layout:
             if order[0] == 'Separator':
                 child = dom.createElement('Separator')
                 node.appendChild(child)
@@ -522,31 +475,28 @@ class MenuEditor(object):
         return element.appendChild(node)
 
     def createLayout(self, items):
-        layout = Layout()
-        layout.order = []
-
-        layout.order.append(['Merge', 'menus'])
+        layout = []
+        layout.append(('Merge', 'menus'))
         for item in items:
-            if isinstance(item, tuple):
-                if item[0] == 'Separator':
-                    layout.parseSeparator()
-                elif item[0] == 'Menu':
-                    layout.parseMenuname(item[1])
-                elif item[0] == 'Item':
-                    layout.parseFilename(item[1])
-            elif isinstance(item, GMenu.TreeDirectory):
-                layout.parseMenuname(item.get_menu_id())
+            if isinstance(item, GMenu.TreeDirectory):
+                layout.append(('Menuname', item.get_menu_id()))
             elif isinstance(item, GMenu.TreeEntry):
-                layout.parseFilename(item.get_desktop_file_id())
+                layout.append(('Filename', item.get_desktop_file_id()))
             elif isinstance(item, GMenu.TreeSeparator):
-                layout.parseSeparator()
-        layout.order.append(['Merge', 'files'])
+                layout.append(('Separator',))
+            else:
+                layout.append(item)
+        layout.append(('Merge', 'files'))
         return layout
 
     def addItem(self, parent, file_id, dom):
         xml_parent = self.getXmlMenu(self.getPath(parent), dom.documentElement, dom)
         self.addXmlFilename(xml_parent, dom, file_id, 'Include')
 
+    def moveItem(self, parent, item, before=None, after=None):
+        self.positionItem(parent, item, before=before, after=after)
+        self.save()
+
     def positionItem(self, parent, item, before=None, after=None):
         contents = self.getContents(parent)
         if after:
@@ -565,7 +515,7 @@ class MenuEditor(object):
             contents.remove(item)
         contents.insert(index, item)
         layout = self.createLayout(contents)
-        dom = self.applications.dom
+        dom = self.dom
         menu_xml = self.getXmlMenu(self.getPath(parent), dom.documentElement, dom)
         self.addXmlLayout(menu_xml, layout, dom)
 
@@ -618,19 +568,3 @@ class MenuEditor(object):
             node.appendChild(self.addXmlTextElement(node, 'Old', final_old, dom))
             node.appendChild(self.addXmlTextElement(node, 'New', new, dom))
             return element.appendChild(node)
-
-class Layout(object):
-    def __init__(self):
-        self.order = []
-
-    def parseMenuname(self, value):
-        self.order.append(['Menuname', value])
-
-    def parseSeparator(self):
-        self.order.append(['Separator'])
-
-    def parseFilename(self, value):
-        self.order.append(['Filename', value])
-
-    def parseMerge(self, merge_type='all'):
-        self.order.append(['Merge', merge_type])
